@@ -88,7 +88,7 @@ class _OSLCOperations_Mixin:
     #
     # sortby is a list of attribute URIs (e.g. dcterms:identifier
     # sortorder default is + for ascending alphabetic sort, use'-' to get descending alphabetic sorting - use '>' to get increasing numeric sorting of the first item in sortby, or < to get decreasing numeric sort (if any value doesn't convert to integer it is assumed to be 0 so will sort first/last)
-    def do_complex_query(self,queryresource, querystring='', searchterms=None, select='', orderby='', properties=None, isnulls=None
+    def do_complex_query(self,queryresource, *, querystring='', searchterms=None, select='', orderby='', properties=None, isnulls=None
                         ,isnotnulls=None, enhanced=True, show_progress=True
                         ,show_info=False, verbose=False, maxresults=None, delaybetweenpages=0.0
                         , pagesize=200
@@ -254,7 +254,7 @@ class _OSLCOperations_Mixin:
 
     # for a query which has been parsed to steps, execute the steps, recursing if there is more than one compount_term
     # a query with two logicalor terms looks like: [[['dcterms:identifier', 'in', [3949]]], [['dcterms:identifier', 'in', [3950]]], 'logicalor']
-    def _evaluate_steps(self, querycapabilityuri,querysteps,resultstack=None, select=None, prefixes=None, orderbys=None, searchterms=None, show_progress=False, verbose=False, maxresults=None, delaybetweenpages=0.0, pagesize=200):
+    def _evaluate_steps(self, querycapabilityuri,querysteps,*,resultstack=None, select=None, prefixes=None, orderbys=None, searchterms=None, show_progress=False, verbose=False, maxresults=None, delaybetweenpages=0.0, pagesize=200):
         logger.info( f"_evaluate_steps {querysteps}" )
         resultstack = resultstack if resultstack is not None else []
         orderbys = orderbys or []
@@ -276,7 +276,7 @@ class _OSLCOperations_Mixin:
 #                    raise Exception( f"Very strange parse result! {step}" )
                 else:
                     # do an actual query
-                    results = self.execute_oslc_query(querycapabilityuri,whereterms=[step], select=select, prefixes=prefixes, orderbys=orderbys, searchterms=searchterms, show_progress=show_progress, maxresults=maxresults, delaybetweenpages=delaybetweenpages, pagesize=pagesize)
+                    results = self.execute_oslc_query(querycapabilityuri,whereterms=[step], select=select, prefixes=prefixes, orderbys=orderbys, searchterms=searchterms, show_progress=show_progress, maxresults=maxresults, delaybetweenpages=delaybetweenpages, pagesize=pagesize, verbose=verbose)
                     if isinstance(results, list):
                         resultlist = {}
                         for result in results:
@@ -321,12 +321,41 @@ class _OSLCOperations_Mixin:
         logger.info( f"{resultstack=}" )
         return resultstack
 
+    # returns the single core artifact from the query results
+    # assumes query results include rm_nav:parent
+    def find_core_artifact(self,queryresults):
+        cas = self.find_core_artifacts(queryresults)
+        if len(list(cas.keys()))>1:
+            raise Exception( "More than one core artifact found when only one expected!" )
+        if len(list(cas.eys()))==0:
+            raise Exception( "No core artifact found!" )
+        return cas
+
+    # returns just the core artifacts (which have rm_nav:parent) from the query results
+    # assumes query results include rm_nav:parent
+    def find_core_artifact(self,queryresults):
+        results = {}
+        for k,v in queryresults.items():
+            if 'rm_nav:parent' in v:
+                results[k]=v
+        return results
+        
+    # returns just the module artifacts (which don't have rm_nav:parent) from the query results
+    # assumes query results would include rm_nav:parent if it had a value
+    def find_module_artifacts(self,queryresults):
+        results = {}
+        for k,v in queryresults.items():
+            if 'rm_nav:parent' not in v:
+                results[k]=v
+        return results
+        
+
     # lower-level OSLC query with prepared arguments
     # by default returns a list of uris as result, but if you provide a select list of attributes, returns a dictionary with as key the artifact uri, each result containing a dictionary with the selected values
     # the whereterms can be created using create_query_operator_string
-    # NOTE that prefixes is keyed by URL and the value is the prefix!
+    # NOTE that prefixes is reversed from what you might expect, i.e. keyed by URL and the value is the prefix!
     # NOTE that whereterms should be a list of lists (the oslc terms) - each of these nested lists is ['attribute',operator',value'] - if more than one and'd term, the first entry must be 'and'!
-    def execute_oslc_query(self, querycapabilityuri, whereterms=None, select=None, prefixes=None, orderbys=None, searchterms=None, show_progress=False, verbose=False, maxresults=None, delaybetweenpages=0.0, pagesize=200):
+    def execute_oslc_query(self, querycapabilityuri, *, whereterms=None, select=None, prefixes=None, orderbys=None, searchterms=None, show_progress=False, verbose=False, maxresults=None, delaybetweenpages=0.0, pagesize=200):
         if select is None:
             select = []
         prefixes = prefixes or {}
@@ -406,7 +435,7 @@ class _OSLCOperations_Mixin:
             where_clauses = self._get_query_clauses(whereterms, uri_to_prefix_map)
         else:
             where_clauses = []
-
+#        print( f"{where_clauses=}" )
         # work out the prefixes to be sent in the oslc query
         theprefixes = []
         allprefixes=[]
@@ -485,12 +514,14 @@ class _OSLCOperations_Mixin:
         params.update(query)
         logger.info( f"The parameters for this query are {params}" )
 
-        fullurl = f"{query_url}?{urllib.parse.urlencode(params)}"
+        fullurl = f"{query_url}?{urllib.parse.urlencode( params, quote_via=urllib.parse.quote, safe='')}"
         if verbose:
             print( f"Full query URL is {fullurl}" )
+#        print( f"Full query URL is {fullurl}" )
 
         # retrieve all pages of results - they will be processed later
         total = 1
+        page = 0
         if show_progress:
             pbar = None
 #            pbar = tqdm.tqdm(initial=0, total=total,smoothing=1,unit=" results",desc="Querying          ")
@@ -502,8 +533,9 @@ class _OSLCOperations_Mixin:
         terminate=False
         while True:
             logger.debug('OSLC Query URI: ' + query_url)
+            page += 1
             # request this page
-            this_result_xml = self.execute_get_rdf_xml(query_url, params=params, headers=headers, cacheable=False)
+            this_result_xml = self.execute_get_rdf_xml(query_url, params=params, headers=headers, cacheable=False, intent=f"Retrieve {utils.nth(page)} page of OSLC query results")
             queryurls.append(query_url)
             # accumulate the results
             result_xmls.append(this_result_xml)
@@ -579,7 +611,7 @@ class _OSLCOperations_Mixin:
                 break
             if delaybetweenpages>0.0:
                 time.sleep(delaybetweenpages)
-
+                
         # finished doing the actual query - now process what's been received!
 
         # if showing progress and pbar has been created (after the first set of results if paged)
@@ -610,24 +642,34 @@ class _OSLCOperations_Mixin:
 
         # check the first set of results to decide what mode we are in
         rdfs_member_es = rdfxml.xml_find_elements( result_xmls[0],'.//rdfs:member/*')
+        # only RM returns rdfs:member with sub-tags
         logger.debug(f"rdfs_member_es={rdfs_member_es}")
         if len(rdfs_member_es) == 0:
+            # non-RM 
             rdfs_member_es = rdfxml.xml_find_elements( result_xmls[0], './/rdfs:member')
             logger.debug(f"rdfs_member_es1={rdfs_member_es}")
             if len(rdfs_member_es) == 0:
-                rdfs_member_es = rdfxml.xml_find_elements( result_xmls[0], './/rdf:Description[@rdf:about]')
-                if len(rdfs_member_es) == 0:
+                # QM and GCM don't return rdfs:member like CM
+                rdfs_member_es = rdfxml.xml_find_elements( result_xmls[0], './/rdf:Description/ldp:contains')
+#                print( f"0 {rdfs_member_es=}" )
+                if len(rdfs_member_es) != 0:
+                    # for GCM, one element holds all the ldp:contains pointing at each result
                     gcmode = True
-                    rdfs_member_es = rdfxml.xml_find_elements( result_xmls[0], './/ldp:contains')
                     logger.info(f"rdfs_member_es2={rdfs_member_es}")
+#                    print(f"rdfs_member_es2={rdfs_member_es}")
                 else:
+                    # for QM, each result has a Description and there is no overall container like GCM
+                    rdfs_member_es = rdfxml.xml_find_elements( result_xmls[0], './/rdf:Description[@rdf:about]')
                     qmmode = True
                     logger.debug(f"rdfs_member_es3={rdfs_member_es}")
             else:
+                # only CM has rdfs:member with no sub-tags
                 cmmode = True
         else:
+            # only RM returns rdfs:member with sub-tags
             rmmode = True
         logger.info(f"cmmode={cmmode} rmmode={rmmode} gcmode={gcmode} {qmmode=}")
+#        print(f"cmmode={cmmode} rmmode={rmmode} gcmode={gcmode} {qmmode=}")
         logger.debug( f"{prefixes=}" )
         revprefixes = { v:k for k,v in prefixes.items()}
         # with select - build a dictionary
@@ -650,8 +692,10 @@ class _OSLCOperations_Mixin:
                 rdfs_member_es = rdfxml.xml_find_elements( result_xml, './/ldp:contains')
             elif qmmode:
                 rdfs_member_es = rdfxml.xml_find_elements( result_xml, './/rdf:Description[@rdf:about]/qm_rqm:orderIndex/..')
+#                print( f"1 {rdfs_member_es=}" )
                 if len(rdfs_member_es)==0:
                     rdfs_member_es = rdfxml.xml_find_elements( result_xml, './/rdf:Description[@rdf:about]/dcterms:title/..')
+#                    print( f"2 {rdfs_member_es=}" )
 
 
             # process them
@@ -698,6 +742,8 @@ class _OSLCOperations_Mixin:
                                     place = rdfxml.remove_tag(ent.tag)+"/"+rdfxml.remove_tag(subent.tag)
 #                                    place = f"{rdfxml.tag_to_prefix(ent.tag)}/{rdfxml.tag_to_prefix(subent.tag)}"
                                     value = subent.text
+                                    if value is None:
+                                        value = rdfxml.xmlrdf_get_resource_uri(subent)
                                     if place in result[about]:
                                         result[about][place].append(value)
                                         logger.debug( f"Saving{about} {place} {value}" )
